/*
 * This file is part of the ESPDA Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * $Id: espda_check.c 162070 2014-06-11 10:18:16Z gcupani $
 * $Author: gcupani $
 * $Date: 2014-06-11 12:18:16 +0200 (Wed, 11 Jun 2014) $
 * $Revision: 162070 $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                   Includes
 -----------------------------------------------------------------------------*/

#include "espda_check.h"

/*----------------------------------------------------------------------------*/
/**
  @defgroup  espda_check  Checks on frames
 
  The names, number, and tags of input frames are checked for consistency.  
 */
/*----------------------------------------------------------------------------*/

/**@{*/

/*----------------------------------------------------------------------------*/
/**
  @brief   Check the names of the frames.
  @param   set      A frame set.
  @param   cfg      The recipe configuration object.
  @param   cfg_tag  The configuration tag.
  @return  CPL error code.
  
  This function scans a frame set @a set and checks if the frames with the 
  expected tags exist. The tags are taken from the recipe configuration object 
  @a cfg, defined with its configuration tag @a cfg_tag. 
  
  If @a set, @a cfg, or @cfg_tag are NULL, an error is issued. If a required 
  frame does not exist, an error is issued. If an optional frame does not exist 
  and verbosity level is greater than 0, a warning is issued.
 */
/*----------------------------------------------------------------------------*/

cpl_error_code espda_check_name(cpl_frameset *set, cpl_recipeconfig *cfg,
    const char *cfg_tag) {
    extern int v_level;
    espda_util_error(set == NULL, CPL_ERROR_NULL_INPUT, cpl_error_get_code());
    espda_util_error(cfg == NULL, CPL_ERROR_NULL_INPUT, cpl_error_get_code());
    espda_util_error(cfg_tag == NULL, CPL_ERROR_NULL_INPUT, 
        cpl_error_get_code());

    // Loop over frames
    cpl_frameset_iterator *iter = cpl_frameset_iterator_new(set);
    for (int i = 0; i < (int)cpl_frameset_get_size(set); i++) {
        const char *name = cpl_frame_get_filename(
            cpl_frameset_iterator_get(iter));
        const char *tag  = cpl_frame_get_tag(cpl_frameset_iterator_get(iter));
            
        // Frames are tentatively open to see if they exist
        FILE *frame = fopen(name, "r");
        if (frame) {
            fclose(frame);
        }
        else {
            int min = cpl_recipeconfig_get_min_count(cfg, cfg_tag, tag);
            
            // Required frame
            if (min > 0) { 
                cpl_error_set_message(cpl_func, CPL_ERROR_FILE_NOT_FOUND, 
                    "Required frame %s with tag %s does not exist.", name, tag);
            }
            
            // Optional frame
            else {
                espda_print_verb(1, cpl_msg_warning(cpl_func, " Optional "
                    "frame %s with tag %s does not exist.", name, tag));
            }
        }
        cpl_frameset_iterator_advance(iter, 1);
    }
    cpl_frameset_iterator_delete(iter);
    if (cpl_error_get_code() != CPL_ERROR_NONE) {
        cpl_recipeconfig_delete(cfg);
    }
    return cpl_error_get_code();
}

/*----------------------------------------------------------------------------*/
/**
  @brief   Check the number of the frames.
  @param   set      A frame set.
  @param   cfg      The recipe configuration object.
  @param   cfg_tag  The configuration tag.
  @return  CPL error code.
  
  This function scans a frame set @a set and checks the number of the frames 
  with the expected tags. The tags are taken from the recipe configuration 
  object @a cfg, defined with its configuration tag @a cfg_tag.
  
  If @a set, @a cfg, or @cfg_tag are NULL, an error is issued. If there are too 
  few required frames with a given tag, an error is issued. If there are no 
  optional frames with a given tag and verbosity level is greater than 0, a 
  warning is issued. 
 */
/*----------------------------------------------------------------------------*/

cpl_error_code espda_check_num(cpl_frameset *set, cpl_recipeconfig *cfg,
    const char *cfg_tag) {
    extern int v_level;
    espda_util_error(set == NULL, CPL_ERROR_NULL_INPUT, cpl_error_get_code());
    espda_util_error(cfg == NULL, CPL_ERROR_NULL_INPUT, cpl_error_get_code());
    espda_util_error(cfg_tag == NULL, CPL_ERROR_NULL_INPUT, 
        cpl_error_get_code());

    char **tag_list = cpl_recipeconfig_get_inputs(cfg, cfg_tag);
    int tag_num = 0;
    
    // Tags are counted
    while (tag_list[tag_num] != NULL) {
        tag_num++;
    }
    
    for (int j = -1; j < tag_num; j++) {
        
        // Minimum and total number of frames for each tag are determined 
        char tag[LEN_STRING];
        if (j == -1) {
            strcpy(tag, cfg_tag);
        }
        else {
            strcpy(tag, tag_list[j]);
        }
        int min = cpl_recipeconfig_get_min_count(cfg, cfg_tag, tag);
        int num = cpl_frameset_count_tags(set, tag);

        // Required frames
        if (num < min) {
            cpl_error_set_message(cpl_func, CPL_ERROR_ILLEGAL_INPUT, "Too few "
                "frames with tag %s (%i): at least %i expected.", tag, num, 
                min);
        }
        
        // Optional frames
        else if (num == min && min == 0) {
            espda_print_verb(1, cpl_msg_warning(cpl_func, " No optional frames "
                "with tag %s.", tag));
        }
    }
    for (int j = 0; j < tag_num; j++) {
        cpl_free(tag_list[j]);
    }    
    cpl_free(tag_list);
    if (cpl_error_get_code() != CPL_ERROR_NONE) {
        cpl_recipeconfig_delete(cfg);
    }
    return cpl_error_get_code();
}

/*----------------------------------------------------------------------------*/
/**
  @brief   Check the tags of the frames.
  @param   set      A frame set.
  @param   cfg      The recipe configuration object.
  @param   cfg_tag  The configuration tag.
  @return  CPL error code.
    
  This function scans a frame set @a set and checks the tags of the frames. The 
  tags are taken from the recipe configuration object @a cfg, defined with its 
  configuration tag @cfg_tag. 
  
  If @a set, @a cfg, or @cfg_tag are NULL, an error is issued. If a tag is wrong 
  the frame is not used. In this case, if verbosity level is greater than 0 a 
  warning is issued, suggesting a possible substitution for the wrong tag. The 
  substitutionis based on a three-way comparison between the wrong tag and the 
  expected tags, and may not always be accurate.
 */
/*----------------------------------------------------------------------------*/

cpl_error_code espda_check_tag(cpl_frameset *set, cpl_recipeconfig *cfg,
    const char *cfg_tag) {
    extern int v_level;
    espda_util_error(set == NULL, CPL_ERROR_NULL_INPUT, cpl_error_get_code());
    espda_util_error(cfg == NULL, CPL_ERROR_NULL_INPUT, cpl_error_get_code());
    espda_util_error(cfg_tag == NULL, CPL_ERROR_NULL_INPUT, 
        cpl_error_get_code());
    cpl_frameset_iterator *iter = cpl_frameset_iterator_new(set);
    for (int i = 0; i < (int)cpl_frameset_get_size(set); i++) {
        const char  *name = cpl_frame_get_filename(
            cpl_frameset_iterator_get(iter));
        const char  *tag = cpl_frame_get_tag(
            cpl_frameset_iterator_get(iter));
        if (strcmp(tag, cfg_tag)) {
	    char  tag_sugg[LEN_STRING];
	    double sugg = 0.0;
        
            // Three-way comparison between wrong and expected tag
            int j = 0;
            char **tag_list = cpl_recipeconfig_get_inputs(cfg, cfg_tag);
            while (tag_list[j] != NULL) {
                char tag_ref[LEN_STRING];
                strcpy(tag_ref, tag_list[j]);
                cpl_free(tag_list[j]);
                if (j == 0 || fabs((double)strcmp(tag, tag_ref)) < sugg) {
                    strcpy(tag_sugg, tag_ref);
                    sugg     = fabs((double)strcmp(tag, tag_ref));
                }
                j++;
            }
            cpl_free(tag_list);
            
            // Suggestion
            if (sugg != 0) {
                espda_print_verb(1, cpl_msg_warning(cpl_func, " Frame %s "
                    "has wrong tag %s. Did you mean %s instead? The frame "
                    "will not be used.", name, tag, tag_sugg));
            }
        }
        cpl_frameset_iterator_advance(iter, 1);
    }
    cpl_frameset_iterator_delete(iter);    
    if (cpl_error_get_code() != CPL_ERROR_NONE) {
        cpl_recipeconfig_delete(cfg);
    }
    return cpl_error_get_code();
}

/**@}*/
