/*
 * This file is part of the ESPDA Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * espda_rhk.c
 *
 *  Created on: Nov 10, 2015
 *      Author: Christophe Lovis
 */

/*
 * $Author: clovis $
 * $Date: 2015-11-10 14:48:13 $
 * $Revision: 1.16 $
 * $Name: $
 */


/*----------------------------------------------------------------------------
                                Includes
 ----------------------------------------------------------------------------*/

#include <espda_rhk.h>

/*----------------------------------------------------------------------------
                              Functions code
 ----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
 @brief Remove the blaze function from an spectrum
 @param extracted_spectrum_table spectrum to be blaze corrected
 @param spectrum_error_table spectrum error table to be corrected
 @param spectrum_quality_table the quality table is untouch, is added in order
to return the whole estructure, spectrum, error and quality tables
 @param blaze blaze function image
 @param extracted_spectrum_table_corrected_RE spectrum corrected
 @param spectrum_error_table_corrected_RE error table corrected
 @param spectrum_quality_table_RE quality map returned
 @return   CPL_ERROR_NONE iff OK
 */
/*----------------------------------------------------------------------------*/

cpl_error_code espda_correct_rhk_blaze(
                        cpl_image *extracted_spectrum_table,
                        cpl_image *spectrum_error_table,
                        cpl_image *spectrum_quality_table,
                        cpl_image *blaze,
                        cpl_image **extracted_spectrum_table_corrected_RE,
                        cpl_image **spectrum_error_table_corrected_RE,
                        cpl_image **spectrum_quality_table_RE)
{
        cpl_image *extracted_spectrum_table_corrected;
        cpl_image *spectrum_error_table_corrected;
        cpl_image *spectrum_quality_table_corrected;

        extracted_spectrum_table_corrected =
                cpl_image_divide_create(extracted_spectrum_table,blaze);

        spectrum_error_table_corrected =
                cpl_image_divide_create(spectrum_error_table,blaze);

    	spectrum_quality_table_corrected =
        cpl_image_duplicate(spectrum_quality_table);

        *extracted_spectrum_table_corrected_RE =
                extracted_spectrum_table_corrected;

        *spectrum_error_table_corrected_RE =
                spectrum_error_table_corrected;

        *spectrum_quality_table_RE =
                spectrum_quality_table_corrected;

        return cpl_error_get_code();
}

cpl_error_code espda_measure_flux(double *wavelength,
				  double *flux,
                                  double *error,
				  double start_wavelength, 	
				  double end_wavelength, 	
                                  int *quality,
				  int S1D_length,
                                  double *pbflux,
                                  double *pbflux_err)
{

        int index_start = 0;
        int index_end = 0;
	double wavelength_sum = 0.0;
	double flux_sum = 0.0;
	double error_sum_squared = 0.0;

        index_start = 0;
        index_end = S1D_length-1;	

        while ((index_start < S1D_length) && 
	       (wavelength[index_start] < start_wavelength)) index_start++;
        while ((index_end >= 0) && 
	       (wavelength[index_end] >= end_wavelength)) index_end--;

	for (int j = index_start; j < index_end; j++) {

		if (!isnan(flux[j]) && !isnan(error[j]))  {
			flux_sum = flux_sum + flux[j];
			error_sum_squared = 
				error_sum_squared + error[j]*error[j];
		}
	}

	*pbflux = flux_sum / (wavelength[index_end] - wavelength[index_start]);	

	*pbflux_err = sqrt(error_sum_squared) / 
		      (wavelength[index_end] - wavelength[index_start]); 

        return cpl_error_get_code();
}

cpl_error_code espda_measure_flux_HK(double *wavelength,
                                  double *flux,
                                  double *error,
                                  double start_wavelength,
                                  double end_wavelength,
				  double central_wavelength,	 
				  double width,	
                                  int *quality,
                                  int S1D_length,
                                  double *pbflux,
                                  double *pbflux_err)
{

        int index_start = 0;
        int index_end = 0;
        double wavelength_sum = 0.0;
        double flux_sum = 0.0;
        double response_sum = 0.0;
        double error_sum_squared = 0.0;
	double response = 0.0;

        index_start = 0;
        index_end = S1D_length-1;	

        while ((index_start < S1D_length) && 
	       (wavelength[index_start] < start_wavelength)) index_start++;
        while ((index_end >= 0) && 
	       (wavelength[index_end] >= end_wavelength)) index_end--;

        for (int j = index_start; j < index_end; j++) {

		if (!isnan(flux[j]) && !isnan(error[j]))  {
			response = 
			(1.-fabs(wavelength[j]-central_wavelength)/width);
			response_sum = response_sum + response;
                	flux_sum = flux_sum + flux[j]*response;
                	error_sum_squared = 
			error_sum_squared + error[j]*error[j]*response*response;
		}
        }

        *pbflux = flux_sum / (wavelength[index_end] - wavelength[index_start]) 
		  / response_sum*(index_end-index_start);

        *pbflux_err = sqrt(error_sum_squared) / 
		      (wavelength[index_end] - wavelength[index_start]) / 
		      response_sum*(index_end-index_start);

	return cpl_error_get_code();
}

cpl_error_code espda_compu_rhk_index(double s,
                                     double s_err,
                                     double bv,
                                     double *rhk,
                                     double *rhk_err)
{

	double ccf,r_phot;

	if((bv > 0.44)&&(bv<1.20)) {

		ccf = 1.13*pow(bv,3.0) - 3.91*pow(bv,2.0) + 2.84*bv - 0.47;	

		if(bv < 0.63) {
			ccf = ccf + 0.135*(0.63-bv) - 
			      0.814*pow(0.63-bv,2.0) + 
                              6.03*pow(0.63-bv,3.0);
		}

		ccf = pow(10.,ccf);
		*rhk = 1.34e-4*ccf*s;
		r_phot = -4.898 + 1.918*pow(bv,2.0) - 2.893*pow(bv,3.0);
		r_phot = pow(10.,r_phot);
		*rhk_err = 1.34e-4*ccf*s_err;

		if((*rhk-r_phot) > 0.) {
			*rhk_err = *rhk_err / (*rhk-r_phot) / log(10.);
			*rhk = log10(*rhk-r_phot);
		} else {
			*rhk_err = 0.;
			*rhk = 0.;
		}

	} else {

		*rhk_err = 0.;
		*rhk = 0.;
	}

        return cpl_error_get_code();
}

cpl_error_code espda_compu_prot_age(double rhk,
                                    double rhk_err,
                                    double bv,
                                    double *prot,
                                    double *prot_err,
                                    double *age,
                                    double *age_err)
{

	double tau;

	if((rhk < -4.3) && (rhk > -5.5)) {
		if(bv < 1.){
			tau = 1.362 - 0.166*(1.-bv) + 0.025*pow(1.-bv,2.0) - 
			      5.323*pow(1.-bv,3.0);
		} else {
			tau = 1.362 - 0.14*(1.-bv);
		}
		*prot = (0.808-2.966*(rhk+4.52))*pow(10.,tau);
		*prot_err = 4.4*bv*1.7-1.7;
	}
	else {
		*prot = 0.;
		*prot_err = 0.;
	}
	if((*prot > 0.) && (bv >= 0.50)){
		*age = 1e-3*pow((*prot/0.407/pow(bv-0.495,0.325)),1./0.566);
		*age_err = 0.05*log(10.)*(*age);
	} else {
		*age = 0.;
		*age_err = 0.;
	}

        return cpl_error_get_code();
}

